@@ -50,6 +50,9 @@
 #include <QWaitCondition>
 #include <qmutex.h>
 #include <QDebug>
+#include <QTimer>
+#include <QMaemo5InformationBox>
+#include <QFile>
 
 using namespace zbar;
 
@@ -73,14 +76,41 @@ BarcodeDetector::~BarcodeDetector() {
 
 }
 
-int BarcodeDetector::analyseImage(char *buffer, char *fourcc, int bpp) {
+int BarcodeDetector::analyseImageSync(char *buffer, int frame_width, int frame_height, char *fourcc, int bpp, int mstimeout, const QObject * receiver, char const* slot_method) {
 
-    if(scanning_status==false)
-        return 0;
+    qDebug() << "In BarcodeDetector::analyseImageSync()";
 
-    if (!this->dmtx_thread.isRunning())
-        dmtx_thread.start();
-    else if(!this->dmtx_thread.isDecoding) {
+    // call once to pass data to the decoder threads
+    this->scanning_status=true; // tell threads they are allowed to decode
+
+    for(int i=0;i<1000;++i){
+        int result = analyseImage(buffer, frame_width, frame_height, fourcc, bpp);
+        //qDebug() << result;
+        if(result){
+            qDebug() << result;
+            break;
+        }
+    }
+    qDebug() << "Done!";
+
+    this->scanning_status=false;
+
+    imgAnalysisSyncTimer.setSingleShot(true);
+    connect(&imgAnalysisSyncTimer, SIGNAL(timeout()), receiver, slot_method);
+    imgAnalysisSyncTimer.start(mstimeout);
+
+    return 0; // no point in returning this
+
+}
+
+
+
+int BarcodeDetector::getAnalysedImageResult(){
+
+    qDebug() << "In BarcodeDetector::getAnalysedImageResult()";
+    qDebug() << "dmtx status, running:" << this->dmtx_thread.isRunning() << ", decoding: " << this->dmtx_thread.isDecoding << "found something: " << this->dmtx_thread.hasFoundBarcode;
+
+    if(this->dmtx_thread.isRunning() && !this->dmtx_thread.isDecoding) {
 
         // check and see if it has decoded something
         if(this->dmtx_thread.hasFoundBarcode){
@@ -91,22 +121,127 @@ int BarcodeDetector::analyseImage(char *buffer, char *fourcc, int bpp) {
 
             this->dmtx_thread.hasFoundBarcode = false; // reset this so we don't look at this data again
 
-            const uchar *buffer = (uchar *)this->dmtx_thread.thread_buffer;
+            const uchar *buf = (uchar *)this->dmtx_thread.thread_buffer;
             unsigned int height = this->dmtx_thread.frame_height;
             unsigned int width = this->dmtx_thread.frame_width;
-            QImage::Format format = QImage::Format_Indexed8; // we can't always assume this.....
-            QImage *qi = new QImage(width, height, format); //(buffer, width, height, width, format);
-            // generate a colourmap
-            qi->setNumColors(256);
-            for(int i=0;i<256;i++){
-                qi->setColor(i,QColor(i,i,i,255).rgba());
-            }
-            qi->fill(0);
-            for(unsigned int i=0;i<width;i++){
-                for(unsigned int j=0;j<height;j++){
-                    qi->setPixel(i,j,buffer[i+j*width]);
-                }
-            }
+            QImage::Format format = QImage::Format_RGB888; // we can't always assume this.....
+            QImage *qi = new QImage(buf, width, height, format); //(buffer, width, height, width, format);
+
+            emit imageAnalysed(TYPE_DMTX,
+                               QString::fromUtf8(this->dmtx_thread.thread_output),
+                               qi); // *output
+
+            return ZBAR_QRCODE; // QR is also 2D so achieves the same effect //BARCODE_TYPE_DMTX;
+        }
+    }
+
+    qDebug() << "zbar status, running:" << this->zbar_thread.isRunning() << ", decoding: " << this->zbar_thread.isDecoding << "found something: " << this->zbar_thread.hasFoundBarcode;
+
+    if(this->zbar_thread.isRunning() && !this->zbar_thread.isDecoding){
+        if(this->zbar_thread.hasFoundBarcode){
+            qDebug() << "analyse_image(): found ZBar barcode, zbar_thread.hasReturned=" << this->zbar_thread.hasFoundBarcode;
+
+            this->zbar_thread.hasFoundBarcode = false; // reset this so we don't look at this data again
+
+            qDebug() << "analyse_image(): found ZBar barcode, zbar_thread.thread_output=" << QString(this->zbar_thread.thread_output);
+            qDebug() << "analyse_image(): found ZBar barcode, zbar_thread.barcode_type=" << QString(this->zbar_thread.barcode_type);
+
+            const uchar *buf = (uchar *)this->zbar_thread.thread_buffer;
+            unsigned int height = this->zbar_thread.frame_height;
+            unsigned int width = this->zbar_thread.frame_width;
+            QImage::Format format = QImage::Format_RGB888; // we can't always assume this.....
+            QImage *qi = new QImage(buf, width, height, format); //(buffer, width, height, width, format);
+
+            emit imageAnalysed(QString::fromUtf8(this->zbar_thread.barcode_type),
+                               QString::fromUtf8(this->zbar_thread.thread_output),
+                               qi); // *output
+
+            return 1;
+        }
+    }
+
+
+    const char *buf = (char *)this->zbar_thread.thread_buffer;
+    unsigned int height = this->zbar_thread.frame_height;
+    unsigned int width = this->zbar_thread.frame_width;
+    int bpp = this->zbar_thread.frame_bpp;
+
+    QImage::Format format = QImage::Format_RGB888; // we can't always assume this.....
+    QImage *qi = new QImage((uchar*)buf, width, height, format); //(buffer, width, height, width, format);
+    qi->save("/home/user/zbarthreadimage.png");
+
+    QFile file("/home/user/zbarthreadimage.raw");
+    file.open(QIODevice::WriteOnly);
+    file.write(buf, height*width*bpp/8); // 4 to make it larger than we want, write all the data
+    file.close();
+
+
+    return 0;
+}
+
+int BarcodeDetector::analyseImage(char *buffer, int frame_width, int frame_height, char *fourcc, int bpp) {
+
+    if(scanning_status==false)
+        return 0;
+
+    if(frame_width>MAX_VIDEO_WIDTH || frame_height>MAX_VIDEO_HEIGHT)
+        return -1; // data too large
+
+    if (!this->dmtx_thread.isRunning()){
+        qDebug() << "Start dmtx thread running";
+
+        // copy the image data to the shared buffer
+        memcpy(this->dmtx_thread.thread_buffer, buffer, frame_width*frame_height*sizeof(char)*bpp/8); // should be the right amount of data....
+        this->dmtx_thread.frame_width=frame_width;
+        this->dmtx_thread.frame_height=frame_height;
+
+        if(strstr((char*)fourcc, "RGB3")){
+            if(bpp==16)
+                this->dmtx_thread.frame_fourcc=DmtxPack16bppRGB;
+            else if (bpp==24)
+                this->dmtx_thread.frame_fourcc=DmtxPack24bppRGB;
+            else if (bpp==32)
+                this->dmtx_thread.frame_fourcc=DmtxPack32bppRGBX;
+        }else if (bpp==8)
+            this->dmtx_thread.frame_fourcc=DmtxPack8bppK;
+
+//            qDebug("is RGB, %d\n", this->dmtx_thread.frame_fourcc); // DEBUG
+
+        this->dmtx_timer = clock();
+
+        dmtx_thread.start();
+
+    }else if(!this->dmtx_thread.isDecoding) {
+
+        // check and see if it has decoded something
+        if(this->dmtx_thread.hasFoundBarcode){
+            qDebug() << "analyse_image(): found datamatrix";
+            // it has decoded something)
+            //strcpy(*output, thread_output); // make output a pointer to the thread shared output buffer
+            //*output = this->dmtx_thread.thread_output;
+
+            this->dmtx_thread.hasFoundBarcode = false; // reset this so we don't look at this data again
+
+            const uchar *buf = (uchar *)this->dmtx_thread.thread_buffer;
+            unsigned int height = this->dmtx_thread.frame_height;
+            unsigned int width = this->dmtx_thread.frame_width;
+
+            QImage::Format format = QImage::Format_RGB888; // we can't always assume this.....
+            QImage *qi = new QImage(buf, width, height, format); //(buffer, width, height, width, format);
+
+//            QImage::Format format = QImage::Format_Indexed8; // we can't always assume this.....
+//            QImage *qi = new QImage(width, height, format); //(buffer, width, height, width, format);
+//            // generate a colourmap
+//            qi->setNumColors(256);
+//            for(int i=0;i<256;i++){
+//                qi->setColor(i,QColor(i,i,i,255).rgba());
+//            }
+//            qi->fill(0);
+//            for(unsigned int i=0;i<width;i++){
+//                for(unsigned int j=0;j<height;j++){
+//                    qi->setPixel(i,j,buf[i+j*width]);
+//                }
+//            }
 
             emit imageAnalysed(TYPE_DMTX,
                                QString::fromUtf8(this->dmtx_thread.thread_output),
@@ -116,7 +251,7 @@ int BarcodeDetector::analyseImage(char *buffer, char *fourcc, int bpp) {
 
         // thread is not running, check to see whether enough time has elapsed to start it running again...
         } else if(clock()-dmtx_timer>=DMTX_RUN_INTERVAL) {
-//            qDebug("analyse_image(): start dmtx thread running");
+            qDebug("analyse_image(): start dmtx thread running");
 
             // copy the image data to the shared buffer
             memcpy(this->dmtx_thread.thread_buffer, buffer, frame_width*frame_height*sizeof(char)*bpp/8); // should be the right amount of data....
@@ -144,6 +279,18 @@ int BarcodeDetector::analyseImage(char *buffer, char *fourcc, int bpp) {
     // otherwise it must be running, so leave it be
 
     if(!this->zbar_thread.isRunning()) {
+        qDebug() << "Start zbar thread running";
+
+        // copy the image data to the shared buffer
+        // set to bpp/24 for now, seems to avoid crashes..
+        memcpy(this->zbar_thread.thread_buffer, buffer, frame_width*frame_height*sizeof(char)*bpp/8); // should be the right amount of data....
+
+        this->zbar_thread.frame_width=frame_width;
+        this->zbar_thread.frame_height=frame_height;
+
+        this->zbar_thread.frame_bpp=bpp;
+        memcpy(this->zbar_thread.frame_fourcc, fourcc, 5);
+
 
         zbar_thread.start();
     } else if(!this->zbar_thread.isDecoding){
@@ -161,21 +308,21 @@ int BarcodeDetector::analyseImage(char *buffer, char *fourcc, int bpp) {
             qDebug() << "analyse_image(): found ZBar barcode, zbar_thread.thread_output=" << QString(this->zbar_thread.thread_output);
             qDebug() << "analyse_image(): found ZBar barcode, zbar_thread.barcode_type=" << QString(this->zbar_thread.barcode_type);
 
-            const uchar *buffer = (uchar *)this->zbar_thread.thread_buffer;
+            const uchar *buf = (uchar *)this->zbar_thread.thread_buffer;
             unsigned int height = this->zbar_thread.frame_height;
             unsigned int width = this->zbar_thread.frame_width;
             QImage::Format format = QImage::Format_RGB888; // we can't always assume this.....
-            QImage *qi = new QImage(buffer, width, height, format); //(buffer, width, height, width, format);
+            QImage *qi = new QImage(buf, width, height, format); //(buffer, width, height, width, format);
 
             emit imageAnalysed(QString::fromUtf8(this->zbar_thread.barcode_type),
                                QString::fromUtf8(this->zbar_thread.thread_output),
                                qi); // *output
 
-            return 1; // not sure this makes any odds anymore anyway.
+            return ZBAR_QRCODE; // not sure this makes any odds anymore anyway.
 
 
         } else { // for the DMTX thread this had a timer, but we want to run as fast as possible so no timer checks
-//            qDebug("analyse_image(): no barcode found, start zbar thread running");
+            qDebug("analyse_image(): no barcode found, start zbar thread running");
 
             // copy the image data to the shared buffer
             // set to bpp/24 for now, seems to avoid crashes..
@@ -204,14 +351,14 @@ int BarcodeDetector::analyseImage(char *buffer, char *fourcc, int bpp) {
 
 }
 
-void BarcodeDetector::setFrameSize(int frameWidth, int frameHeight) {
-    this->frame_width = frameWidth;
-    this->frame_height = frameHeight;
-    this->zbar_thread.frame_width = frameWidth;
-    this->zbar_thread.frame_height = frameHeight;
-    this->dmtx_thread.frame_width = frameWidth;
-    this->dmtx_thread.frame_height = frameHeight;
-}
+//void BarcodeDetector::setFrameSize(int frameWidth, int frameHeight) {
+//    this->frame_width = frameWidth;
+//    this->frame_height = frameHeight;
+//    this->zbar_thread.frame_width = frameWidth;
+//    this->zbar_thread.frame_height = frameHeight;
+//    this->dmtx_thread.frame_width = frameWidth;
+//    this->dmtx_thread.frame_height = frameHeight;
+//}
 
 QImage *BarcodeDetector::getLastImage() {
 
