@@ -50,6 +50,7 @@
 #include "zbardecoderthread.h"
 #include <QWaitCondition>
 #include <QImage>
+#include <QTimer>
 #include <qmutex.h>
 
 #define DMTX_RUN_INTERVAL 1
@@ -66,14 +67,17 @@ public:
 
     // from decoder.c
     // TEMPORARILY made static!
-    int analyseImage(char *buffer, char *fourcc, int bpp);
+    int analyseImageSync(char *buffer, int frame_width, int frame_height, char *fourcc, int bpp, int mstimeout, const QObject * receiver, char const* slot_method);
+    int analyseImage(char *buffer, int width, int height, char *fourcc, int bpp);
     QImage* getLastImage();
-    void setFrameSize(int frameWidth, int frameHeight);
+    //void setFrameSize(int frameWidth, int frameHeight);
 
     bool scanning_status;
 
-    int frame_width;
-    int frame_height;
+    //int frame_width;
+    //int frame_height;
+
+    int getAnalysedImageResult();
 
 signals:
     void imageAnalysed(QString imageType, QString imageData, QImage *image);
@@ -102,7 +106,7 @@ private:
     int deviceType;
 
 
-
+    QTimer imgAnalysisSyncTimer;
 
     DMTXDecoderThread dmtx_thread;
     ZBarDecoderThread zbar_thread;
