@@ -8,13 +8,16 @@
 
 #include "common.h"
 
+#include <QFile>
+#include <QImage>
+
 using namespace std;
 
 DMTXDecoderThread::DMTXDecoderThread()
 {
     this->frame_width = WIDTH; // not really necessary as these won't change during the program
     this->frame_height = HEIGHT;
-    this->thread_buffer = (char*)malloc(this->frame_width*this->frame_height*sizeof(char)*4); // need to account for bpp here, but let's assume 32bit pp is the maximum
+    this->thread_buffer = (char*)malloc(MAX_VIDEO_WIDTH*MAX_VIDEO_HEIGHT*sizeof(char)*4); // need to account for bpp here, but let's assume 32bit pp is the maximum
 
     thread_output=0;
     hasFoundBarcode=false;
@@ -63,6 +66,23 @@ void DMTXDecoderThread::run()
 /* Analyse the image data */
 int DMTXDecoderThread::libdmtx_analyse_image()
 {
+
+//    const char *buf = (char *)this->thread_buffer;
+//    unsigned int height = this->frame_height;
+//    unsigned int width = this->frame_width;
+//    int bpp = 24; //this->frame_bpp;
+
+//    QImage::Format format = QImage::Format_RGB888; // we can't always assume this.....
+//    QImage *qi = new QImage((uchar*)buf, width, height, format); //(buffer, width, height, width, format);
+//    qi->save("/home/user/dmtxthreadimage.png");
+
+//    QFile file("/home/user/dmtxthreadimage.raw");
+//    file.open(QIODevice::WriteOnly);
+//    file.write(buf, height*width*bpp/8); // 4 to make it larger than we want, write all the data
+//    file.close();
+
+
+
     //GdkPixbuf *pixbuf = NULL;
     //GError *error = NULL;
     //unsigned int bpp;
