@@ -41,7 +41,7 @@
 #include <QDebug>
 #include <QFileDialog>
 #include <QFileInfo>
-#include <QMaemo5InformationBox>
+
 
 // gstreamer includes
 #include <gst/gst.h>
@@ -61,7 +61,7 @@
 #include "pluginsettings.h"
 #include "recentdialog.h"
 #include "pluginloaderthread.h"
-
+#include <cmath>
 
 extern QTime debugSpeedTimer;
 
@@ -89,6 +89,8 @@ MaemoBarcodeWindow::MaemoBarcodeWindow(QWidget *parent)
     recentDialog = NULL;
     pluginsLoaded = false;
 
+    imageBuffer = NULL;
+
     // first of all, disable the camera app:
     qDebug() << "MainWindow::MainWindow(): Killing the camera app, t=" << debugSpeedTimer.elapsed();
     QProcess process;
@@ -302,7 +304,7 @@ void MaemoBarcodeWindow::ensurePluginsLoaded()
 {
     if (!pluginsLoaded)
     {
-        QMaemo5InformationBox::information (this, "Loading plugins (first time only)...", 10000);
+        QMaemo5InformationBox::information (this, "Loading plugins (first time only)...", 3000);
         loadPlugins();
         //pluginloaderthread->wait();
         QMaemo5InformationBox::information (this, "Loaded plugins", 1);
@@ -319,12 +321,12 @@ void MaemoBarcodeWindow::loadPlugins() {
     if (pluginsLoaded) // if we were too late and ensurePluginsLoaded() was called...
         return;
 
-    qDebug() << "MainWindow::MainWindow(): Initializing results window, t=" << debugSpeedTimer.elapsed();
+    qDebug() << "loadPlugins(): Initializing results window, t=" << debugSpeedTimer.elapsed();
     resultsWindow = new ResultsWindow(this); // create a resultswindow
 
     qApp->processEvents();
 
-    qDebug() << "Loading plugins..., t=" << debugSpeedTimer.elapsed();
+    qDebug() << "loadPlugins(): Loading plugins..., t=" << debugSpeedTimer.elapsed();
 
     //    pluginsDir = QDir(qApp->applicationDirPath());.
     QDir pluginsDir = QDir("/usr/share/mbarcode/");
@@ -356,6 +358,42 @@ void MaemoBarcodeWindow::loadPlugins() {
 
     qApp->processEvents();
 
+    QList<QString> origPluginNames;
+    //QList<int> origPluginPriorities;
+    QList<bool> origPluginEnabled;
+    //QList<bool> origPluginPresent;
+
+    int numPlugins = settings->beginReadArray("plugins");
+    qDebug() << "loadPlugins(): Loading plugin info, found " << numPlugins;
+
+    //int max_pri=-1;
+    //int pri;
+
+    if(numPlugins>0){
+        for (int i=0; i<numPlugins; ++i){
+            settings->setArrayIndex(i);
+
+            origPluginNames.append(settings->value("pluginName").toString());
+//            pri = settings->value("pluginPriority").toInt(); // read the saved priority
+
+//            while(origPluginPriorities.contains(pri)) // keep incrementing this value until we find a slot that is unused
+//                pri++;
+
+//            if(pri>max_pri)
+//                max_pri = pri; // save for later when we add new plugins
+
+//            origPluginPriorities.append(pri );
+            origPluginEnabled.append(settings->value("pluginEnabled").toBool());
+            //origPluginPresent.append(false); // start with false, then we will look later on
+
+            //qDebug() << "loadPlugins(): Loading plugin settings data for item " << settings->value("pluginName").toString() << " " << pri << " " << settings->value("pluginEnabled").toBool();
+            qDebug() << "loadPlugins(): Loading plugin settings data for item " << settings->value("pluginName").toString() << settings->value("pluginEnabled").toBool();
+        }
+    }
+    settings->endArray();
+
+    this->NumEnabledPlugins=0;
+
     foreach (QString fileName, allPluginFiles) { // go through all files in the plugin dir
         qDebug(qPrintable("Loading " + fileName));
         QPluginLoader loader(fileName); // create a loader for the shared library
@@ -368,88 +406,55 @@ void MaemoBarcodeWindow::loadPlugins() {
             pluginInterfaces.append(interface); // add this interface to our list of interfaces
             interface->initInterface(this); // initialize the plugin
             foreach(PluginAction* pluginAction, interface->getPluginActions()) { // get the list of sink plugins in the interface
-                qDebug(qPrintable("Loading sink plugin " + pluginAction->getName()));
+
+                QString pluginIdentifier(interface->getName() + ": " + pluginAction->getName());
+                // this matches the string shown in the pluginsettings window - see the initialisation fn
+
+                // see if we know about this plugin
+                int ind = origPluginNames.indexOf(pluginIdentifier);
+                if (ind>-1){ // for some reason this is never true
+                    // yes we know about this one
+
+                    //pluginAction->setPriority(origPluginPriorities.value(ind));
+                    pluginAction->setEnabled(origPluginEnabled.value(ind));
+
+                    qDebug() << "loadPlugins(): Found plugin listed in settings " << pluginIdentifier << pluginAction->getEnabled();
+
+                }else{
+                    // no we don't know about this one
+
+                    //pluginAction->setPriority(++max_pri);
+                    pluginAction->setEnabled(true);
+
+                    //qDebug() << "loadPlugins(): Found plugin not listed in settings " << pluginIdentifier << " " << max_pri << " true";
+                    qDebug() << "loadPlugins(): Found plugin not listed in settings " << pluginIdentifier << " true";
+                }
+
+                if (pluginAction->getEnabled())
+                    this->NumEnabledPlugins++;
+
                 pluginActions.append(pluginAction); // add them to our list
             }
             qApp->processEvents();
         } else {
-            qWarning(qPrintable("Could not load: " + fileName + " due to error: " + loader.errorString()));
+            qWarning(qPrintable("loadPlugins(): Could not load: " + fileName + " due to error: " + loader.errorString()));
         }
     }
+
+    //qSort(pluginActions.begin(), pluginActions.end()); // sort the array, still a bit worried about this
+
+    // now make the priorities list contiguous
+    //for(int i=0; i<pluginActions.size(); ++i){
+    //    pluginActions.at(i)->setPriority(i);
+    //}
+
     qDebug() << "Done loading plugins, t=" << debugSpeedTimer.elapsed();
     resultsWindow->setPluginActions(pluginActions); // let the resultwindow have all our plugins
+
     pluginsLoaded = true;
 }
 
 
-//void MaemoBarcodeWindow::loadPlugins() {
-//    // TODO: Create a system which lets the user enable/disable plugins. These should not be loaded.
-//
-//    qDebug("Loading plugins...");
-//
-//    //    pluginsDir = QDir(qApp->applicationDirPath());.
-//    QDir pluginsDir = QDir("/usr/share/mbarcode/");
-//    QDir pluginsDirHome = QDir(QDir::homePath() + "/.config/mbarcode/plugins");
-//
-//    // some path stuff derived from Nokia's plugin tutorials - not really necessary in a Maemo app, but we'll just leave them here
-//#if defined(Q_OS_WIN)
-//    if (pluginsDir.dirName().toLower() == "debug" || pluginsDir.dirName().toLower() == "release")
-//        pluginsDir.cdUp();
-//#elif defined(Q_OS_MAC)
-//    if (pluginsDir.dirName() == "MacOS") {
-//        pluginsDir.cdUp();
-//        pluginsDir.cdUp();
-//        pluginsDir.cdUp();
-//    }
-//#endif
-//    pluginsDir.cd("plugins");
-//    QStringList allPluginFiles;
-//
-//    foreach (QString fileName, pluginsDir.entryList(QDir::Files)) { // go through all files in the plugin dir
-//        allPluginFiles << pluginsDir.absoluteFilePath(fileName);
-//    }
-//    foreach (QString fileName, pluginsDirHome.entryList(QDir::Files)) { // go through all files in the plugin dir in home
-//        allPluginFiles << pluginsDirHome.absoluteFilePath(fileName);
-//    }
-//    foreach (QString fileName, allPluginFiles) { // go through all files in the plugin dir
-//        qDebug(qPrintable("Loading " + fileName));
-//        QPluginLoader loader(fileName); // create a loader for the shared library
-//        QObject *plugin = loader.instance();  // load it as a QObject
-//        if (plugin) { // if it is a plugin (this fails if virtual functions are not implemented correct)
-//            qDebug(qPrintable("Loaded successfully: " + fileName));
-//            pluginFileNames += fileName; // list all our plugin file names
-//            PluginInterface* interface = qobject_cast<PluginInterface*>(plugin); // cast it to our kind of interface
-//            qDebug(qPrintable("Interface name: " + interface->getName()));
-//            pluginInterfaces.append(interface); // add this interface to our list of interfaces
-//            interface->initInterface(this); // initialize the plugin
-//            foreach(PluginAction* pluginAction, interface->getPluginActions()) { // get the list of sink plugins in the interface
-//                qDebug(qPrintable("Loading sink plugin " + pluginAction->getName()));
-//                pluginActions.append(pluginAction); // add them to our list
-//            }
-//        } else {
-//            qWarning(qPrintable("Could not load: " + fileName));
-//        }
-//    }
-//    qDebug("Done loading plugins");
-//    resultsWindow->setPluginActions(pluginActions); // let the resultwindow have all our plugins
-//}
-
-//void MaemoBarcodeWindow::on_btnScan_clicked()
-//{
-//    startScan();
-//}
-
-//void MaemoBarcodeWindow::on_btnCancel_clicked()
-//{
-//    this->videoWidget->stopTimedScan();
-//    videoWidget->stop(true);
-//}
-
-//void MaemoBarcodeWindow::on_btnRefocus_clicked()
-//{
-//    videoWidget->refocus();
-//}
-
 void MaemoBarcodeWindow::dataAnalysed(QString datatype, QString barcodeData, QHash<QString, QString> data) {
     // this is a slot for all data which may come from other plugins
 
@@ -504,13 +509,16 @@ void MaemoBarcodeWindow::showSettingsWindow()
     this->videoWidget->stopTimedScan();
     videoWidget->stop(false);
     qDebug() << "MainWindow::showSettingsWindow(): Creating settings window";
+    ensurePluginsLoaded();
     SettingsWindow *settingsWindow = new SettingsWindow(this);
     settingsWindow->show();
 }
 
 void MaemoBarcodeWindow::OpenAndScanImage()
 {
-    return;
+
+    this->videoWidget->stopTimedScan();
+    videoWidget->stop(false);
 
     QString fileName = QFileDialog::getOpenFileName(this, tr("Open Image"), "/home/user/MyDocs/.images/", tr("Image Files (*.png *.jpg *.bmp *.gif *.tif*)"));
 
@@ -521,32 +529,106 @@ void MaemoBarcodeWindow::OpenAndScanImage()
 
     QImage img(qfi.absoluteFilePath());
 
+
+    // see if we are too large for the thread buffers
+    if (img.width()>MAX_VIDEO_WIDTH && img.height()>MAX_VIDEO_HEIGHT) // both too big, so see which one we should scale to bring both down to correct size
+        if (float(img.width())/float(MAX_VIDEO_WIDTH) > float(img.height())/float(MAX_VIDEO_HEIGHT))
+            img = img.scaledToWidth(MAX_VIDEO_WIDTH,Qt::SmoothTransformation);
+        else
+            img = img.scaledToWidth(MAX_VIDEO_HEIGHT,Qt::SmoothTransformation);
+    else if (img.width()>MAX_VIDEO_WIDTH)
+        img = img.scaledToWidth(MAX_VIDEO_WIDTH,Qt::SmoothTransformation);
+    else if (img.height()>MAX_VIDEO_HEIGHT)
+        img = img.scaledToHeight(MAX_VIDEO_HEIGHT,Qt::SmoothTransformation);
+
     // convert to an appropriate format
+    //img = img.convertToFormat(QImage::Format_RGB888);
     img = img.convertToFormat(QImage::Format_RGB888);
 
     char fourcc[5] = "RGB3";
     int bpp = 24;
+//    char fourcc[5] = "Y800";
+//    int bpp = 8;
 
-    // check the frame size, we might need to change the value of the globals
-    // to reflect this
 
-    int old_height = decoder->frame_height;
-    int old_width = decoder->frame_width;
 
-    decoder->frame_height = img.height();
-    decoder->frame_width = img.width();
+    int padding;
 
-    decoder->scanning_status=true;
-    int type = decoder->analyseImage((char *)img.constBits(), fourcc, bpp);
-    decoder->scanning_status=false;
+    // we'll pad the image to a 4byte multiple of width
+    // this is necessary for the libdmtx code which converts the buffer into a QImage to change its format
+    // it also means we can then look at the returned buffer more easily too!
 
-    decoder->frame_height = old_height;
-    decoder->frame_width = old_width;
+    if(img.bytesPerLine() == bpp/8*img.width()){
+        // we're already a round number, so we're fine
+        padding = 0;
+    }else{            
+        padding=0;
+        int remainingBytes = img.bytesPerLine()-bpp/8*img.width();
+        while( ( padding*bpp/8 - remainingBytes) % 4)
+            padding++; // iterate until we've added enough padding pixels to be divisible by 4 (bytes)
 
-    //videoWidget->useFilesrc(fileName);
-    //videoWidget->start();
+        qDebug() << "bytesPerLine=" << img.bytesPerLine() << "bpp=" << bpp << "remainingBytes=" << remainingBytes << "padding=" << padding << "old width" << img.width() << "new width=" << img.width()+padding;
+    }
+    int width = img.width()+padding; // this is the padded width
 
-    // but how do we stop the pipeline? Or does it stop itself once it's processed the image, probably
+
+
+    // the line data in the QImage are aligned on 32bit boundaries
+    // but the decoder expects a contiguous chunk of data
+    // so we need to generate that
+    if(imageBuffer)
+        free(imageBuffer);
+
+    imageBuffer = (uchar*)malloc(width*img.height()*bpp/8*sizeof(char));
+    if(!imageBuffer)
+        return; // memory allocation error
+
+
+    uchar* pos = imageBuffer;
+    for(int i=0; i<img.height(); ++i){
+        memcpy(pos, img.constScanLine(i), img.width()*bpp/8); //img.bytesPerLine()
+        pos = pos + img.width()*bpp/8;
+
+        // add padding pixels
+        for(int j =0; j<padding; ++j){ // iterate through the padding pixels
+            for(int k=0;k<bpp/8; ++k){ // iterate though the components of the pixel
+                *pos = 0; // give the pixel component a value
+                pos++; // move to the next one
+            }
+        }
+    }
+
+//    qDebug() << "Image data w=" << img.width() << "height " << img.height() << "bytes per line " << img.bytesPerLine() << "bytecount " << img.byteCount();
+
+//    // debugging, as we don't seem to be able to decode
+//    QFile file("/home/user/mbarcode-image.raw");
+//    file.open(QIODevice::WriteOnly);
+//    file.write((const char*)(imageBuffer), img.height()*width*bpp/8);
+//    file.close();
+
+//    img.save("/home/user/mbarcode-image.png");
+
+
+
+    int type = decoder->analyseImageSync((char *)imageBuffer, width, img.height(), fourcc, bpp, 1000, this, SLOT(analyseImageSyncCallBack()));
+
+    free(imageBuffer);
+    imageBuffer=0;
+}
+
+void MaemoBarcodeWindow::analyseImageSyncCallBack(){
+    decoder->scanning_status=false; // stop decoder threads if they've not stopped already
+    int result = decoder->getAnalysedImageResult();
+
+    if(imageBuffer)
+        free(imageBuffer);
+
+    qDebug() << "In MaemoBarcodeWindow::analyseImageSyncCallBack()";
+
+    if(result)
+        QMaemo5InformationBox::information (this, "Barcode detected", 1000);
+    else
+        QMaemo5InformationBox::information (this, "No barcode detected", 1000);
 
 }
 
