@@ -12,17 +12,17 @@
   is called, which in turn calls the plugins class.
 */
 
-class SinkPlugin;
+class PluginAction;
 
 class PluginIntermediate : public QObject
 {
     Q_OBJECT
 public:
-    PluginIntermediate(SinkPlugin *plugin, QWidget *parentWindow);
+    PluginIntermediate(PluginAction *plugin, QWidget *parentWindow);
 public slots:
     void clickedPlugin();
 private:
-    SinkPlugin *plugin;
+    PluginAction *plugin;
     QWidget *parentWindow;
 };
 
