@@ -40,14 +40,16 @@
 #include "pluginintermediate.h"
 #include <QTimer>
 #include <QItemSelectionModel>
+#include "maemobarcodewindow.h"
 
 #define PLUGIN_COUNT_LIMIT    6
 
-ResultsWindow::ResultsWindow(QWidget *parent) :
+ResultsWindow::ResultsWindow(MaemoBarcodeWindow *parent) :
         QMainWindow(parent),
         ui(new Ui::ResultsWindow)
 {
     ui->setupUi(this);
+    this->parent = parent;
     checkTimer = new QTimer(this);
     connect(checkTimer, SIGNAL(timeout()),this,SLOT(checkPlugins()));
     setAttribute(Qt::WA_Maemo5StackedWindow);
@@ -56,6 +58,8 @@ ResultsWindow::ResultsWindow(QWidget *parent) :
 
     selectionTimer = new QTimer(this);
     connect(selectionTimer, SIGNAL(timeout()), this, SLOT(selectionTimerExpiry()));
+
+    checkPlugins(); // call it here to avoid the initial wait
 }
 
 ResultsWindow::~ResultsWindow()
@@ -115,14 +119,19 @@ void ResultsWindow::checkPlugins() {
 
     foreach(PluginAction* plugin, plugins) {
 //        qDebug() << "Checking action plugin " << plugin->getName() << "is ready" << plugin->isReady();
-        if(plugin->isReady() && !addedPlugins.contains(plugin)) { // check if it's ready and that it has not yet been added
+        if(plugin->getEnabled() && plugin->isReady() && !addedPlugins.contains(plugin)) { // check if it's ready and that it has not yet been added
             addedNewPlugins=true;
             addedPlugins.append(plugin); // add it to the list of added plugins, so we don't add it twice
 //            qDebug() << "Added plugin" << plugin->getText() << plugin->getName();
         }
     }
 
-    if(addedNewPlugins==true){
+    if(addedNewPlugins==true && this->parent->NumEnabledPlugins==1){
+        this->hide(); // hid this window
+        checkTimer->stop(); // no point in continuing to check for plugins
+        addedPlugins.at(0)->clickAction(this); // run the plugin
+        return;
+    }else if(addedNewPlugins==true){
         QAbstractItemModel *oldModel = ui->tblResults->model();
         PluginActionListModel *model = new PluginActionListModel(this, addedPlugins);
         ui->tblResults->setModel(model);
