@@ -6,6 +6,9 @@
 #include <zbar.h>
 #include <iostream>
 
+#include <QFile>
+#include <QImage>
+
 #include "zbardecoderthread.h"
 #include "common.h"
 
@@ -15,7 +18,7 @@ ZBarDecoderThread::ZBarDecoderThread()
 {
     this->frame_width = WIDTH; // not really necessary as these won't change during the program
     this->frame_height = HEIGHT;
-    this->thread_buffer = (char*)malloc(this->frame_width*this->frame_height*sizeof(char)*4); // need to account for bpp here, but let's assume 32bit pp is the maximum
+    this->thread_buffer = (char*)malloc(MAX_VIDEO_WIDTH*MAX_VIDEO_HEIGHT*sizeof(char)*4); // need to account for bpp here, but let's assume 32bit pp is the maximum
 
     thread_output=0;
     hasFoundBarcode=false;
@@ -26,7 +29,7 @@ ZBarDecoderThread::ZBarDecoderThread()
     cout << "Created scanner" << endl;
     /* configure the reader */
     //zbar_image_scanner_set_config(scanner, NULL, ZBAR_CFG_ENABLE, 1);
-    zbar_image_scanner_enable_cache(this->scanner, TRUE);
+    //zbar_image_scanner_enable_cache(this->scanner, TRUE);
     cout << "Enabled scanner" << endl;
     this->image = zbar_image_create();
     cout << "Created image" << endl;
@@ -80,6 +83,23 @@ void ZBarDecoderThread::run()
 int ZBarDecoderThread::libzbar_analyse_image()
 {
 
+//    const char *buf = (char *)this->thread_buffer;
+//    unsigned int height = this->frame_height;
+//    unsigned int width = this->frame_width;
+//    int bpp = this->frame_bpp;
+
+//    QImage::Format format = QImage::Format_RGB888; // we can't always assume this.....
+//    QImage *qi = new QImage((uchar*)buf, width, height, format); //(buffer, width, height, width, format);
+//    qi->save("/home/user/zbarthreadimage.png");
+
+//    QFile file("/home/user/zbarthreadimage.raw");
+//    file.open(QIODevice::WriteOnly);
+//    file.write(buf, height*width*bpp/8); // 4 to make it larger than we want, write all the data
+//    file.close();
+
+
+
+
     zbar_image_set_format(image, *(int*)frame_fourcc); //*(int*)"RGB3");
     zbar_image_set_size(image, frame_width, frame_height);
     zbar_image_set_data(image, thread_buffer, frame_width * frame_height * frame_bpp/8, NULL); //zbar_image_free_data
